/*-----------------------------------------------------------
						COMPLEX.H
-----------------------------------------------------------*/
#include	<math.h>

#define	topexp		150
#define	min_num		1.e-150
#define	min_num2	1.e-300
#define	max_num		1.e150
#define	max_num2	1.e300
#define M_E         2.71828182845904523536
#define M_PI        3.14159265358979323846
#define M_2_PI      6.28318530717958647693
#define M_PI_2      1.57079632679489661923
#define M_PI_4      0.785398163397448309616

#ifdef	_RAD_INIT
	int	rad = 0;
#else
	extern	int rad;
#endif

// ǉ@2000/06/09
struct complex{
	double	x, y;
};

class	complx
{
	private:
		double	x, y;
	public:
		///// RXgN^
		complx( double	r = 0, double	i = 0 )	{x = r;	y = i;}
		complx( complex a )	{x = a.x;	y = a.y;}

		///// Aɕ
		double	Real()	{ return x; }
		double	Image()	{ return y; }
//		double	Fnc(double x)	{ return	exp(-x*x/2); }


		/////  {[~
		friend	complx	operator	-(complx	a)
		{	return complx( -a.x, -a.y ); }

		friend	complx	operator	*(complx	a, complx	b)
		{	return	complx(a.x * b.x - a.y * b.y, a.x * b.y + a.y * b.x);}

		friend	complx	operator	*(complx	a, double	b)
		{	return	complx(a.x * b, a.y * b);}

		friend	complx	operator	*(double	b, complx	a)
		{	return	complx(a.x * b, a.y * b);}

		friend	complx	operator	+(complx	a, double	b)
		{	return	complx(a.x + b, a.y);}

		friend	complx	operator	+(double	b, complx	a)
		{	return	complx(a.x + b, a.y);}

		friend	complx	operator	+(complx	a, complx	b)
		{	return	complx(a.x + b.x, a.y + b.y);}

		friend	complx	operator	/(complx	a, double	b)
		{
			if(fabs(b) < min_num){
				b = (b > 0.0)?1.:-1.;
				return	complx(a.x*b*max_num, a.y*b*max_num);
			}
			return	complx(a.x / b, a.y / b);
		}

		friend	complx	operator	/(double	b, complx	a)
		{
			double	c, d, e;
			d = (a.x > 0)?a.x:-a.x;	e = (a.y > 0)?a.y:-a.y;
			if((d > max_num) || (e > max_num)){
				c = min_num2;
				return	complx(b*a.x*c, -b*a.y*c);
			}
			if((d < min_num) && (e < min_num)){
				c = max_num2;
				return	complx(b*a.x*c, -b*a.y*c);
			}
			c = a.x*a.x + a.y*a.y;
			return	complx(b*a.x/c, -b*a.y/c);
		}

		friend	complx	operator	/(complx	a, complx	b)
		{
			double	c, e, f;
			complx	d;
			if((a.x == 0.0) && (a.y == 0.0))	return	complx(0.0, 0.0);
			if((b.x == 0.0) && (b.y == 0.0)){
				c = max_num2;
				d.x = ((a.x > 0.0)?c:-c);
				d.y = ((a.y > 0.0)?c:-c);
				return	d;
			}
			e = (b.x > 0)?b.x:-b.x;	f = (b.y > 0)?b.y:-b.y;
			if((e > max_num) || (f > max_num)){
				c = min_num2;
				e = (b.x > 0)?c:-c;	f = (b.y > 0)?c:-c;
				d.x = e * a.x + f * a.y;
				d.y = e * a.y - f * a.x;
				return	d;
			}
			if((e < min_num) && (f < min_num)){
				c = max_num2;
				e = (b.x > 0)?c:-c;	f = (b.y > 0)?c:-c;
				d.x =	e * a.x + f * a.y;
				d.y = e * a.y - f * a.x;
				return	d;
			}
			c = b.x*b.x + b.y*b.y;
			d.x = (a.x * b.x + a.y * b.y)/c;
			d.y = (a.y * b.x - a.x * b.y)/c;
			return	d;
		}

		friend	complx	operator	-(complx	a, double	b)
		{	return	complx(a.x - b, a.y);}

		friend	complx	operator	-(double	b, complx	a)
		{	return	complx(b - a.x, -a.y);}

		friend	complx	operator	-(complx	a, complx	b)
		{	return	complx(a.x - b.x, a.y - b.y);}


		///// ֐
		friend	complx	real( complx a )
		{	complx buff( a.x, 0 );	return	buff;	}

		friend	complx	imag( complx a )
		{	complx buff( a.y, 0 );	return	buff;	}

		friend	complx	abs(complx	a)
		{
			if((fabs(a.x) < min_num) && (fabs(a.y) < min_num))	return	0.0;
			complx buff( sqrt(a.x * a.x + a.y * a.y), 0 );
			return	buff;
		}

		friend	complx	norm(complx	a)
		{
			if((fabs(a.x) < min_num) && (fabs(a.y) < min_num))	return	0.0;
			complx buff( sqrt(a.x * a.x + a.y * a.y), 0 );
			return	buff;
		}

		friend	complx	norm2(complx	a)
		{
			if((fabs(a.x) < min_num) && (fabs(a.y) < min_num))	return	0.0;
			complx buff( a.x * a.x + a.y * a.y, 0 );
			return	buff;
		}

		friend	complx	db(complx	a)
		{
			complx buff( sqrt(a.x * a.x + a.y * a.y), 0 );
			return	log10(buff) * 20.;
		}

		friend	complx	undb(complx	a)
		{
			return	pow(10., a.x / 20);
		}

		friend	complx	unpow(complx	a, complx	b)
		{
			return	log(a)/log(b);
		}

		friend	complx	rtpow(complx	a, complx	b)
		{
			return	pow(a, 1/b);
		}

		friend	complx	conj(complx	a)
		{
			a.y = -a.y;
			return	a;
		}

		friend	complx	arg(complx	a)
 		{
 			if(rad)	return complx( atan2(a.y, a.x), 0 );
 			else	return complx( atan2(a.y, a.x) * 180 / M_PI, 0 );
 		}

		friend	complx	exp(complx	a)
		{
			double	sc;
			sc = a.x;
			if(sc > topexp)	return		complx(max_num * cos(a.y), max_num * sin(a.y));
			if(sc < -topexp)	return	complx(0., 0.);
			sc = exp(sc);
			return	complx(sc*cos(a.y), sc*sin(a.y));
		}

		friend	complx	log(complx	a)
		{
			double	ud;
			if((fabs(a.x) < min_num) && (fabs(a.y) < min_num)){
				return	complx(min_num, 0.0);
			}
			if((fabs(a.x) > max_num) || (fabs(a.y) > max_num))	ud = max_num2;
			else	ud = a.x * a.x + a.y * a.y;
			return	complx((log(ud))/2, atan2(a.y, a.x));
		}

		friend	complx	log10(complx	a)
		{
			complx	c;
			double	ud, ud2;
			if((fabs(a.x) < min_num) && (fabs(a.y) < min_num)){
				return	complx(min_num, 0.0);
			}
			if((fabs(a.x) > max_num) || (fabs(a.y) > max_num))	ud = max_num2;
			else	ud = a.x * a.x + a.y * a.y;
			ud2 = 10.0;
			ud = log(ud2);
			c = log(a)/ud;
			return	c;
		}

		friend	complx	sin(complx	a)
		{
			double	x, y;
			complx	d;
			if(rad)	x = a.x;
			else	x = a.x / 180. * M_PI;
			y = a.y;
			d.x = sin(x) * cosh(y);
			d.y = cos(x) * sinh(y);
			return	d;
		}

		friend	complx	cos(complx	a)
		{
			double	x, y;
			complx	d;
			if(rad)	x = a.x;
			else	x = a.x / 180. * M_PI;
			y = a.y;
			d.x = cos(x) * cosh(y);
			d.y = -sin(x) * sinh(y);
			return	d;
		}

		friend	complx	tan(complx	a)
		{
			return	sin(a) / cos(a);
		}

		friend	complx	sinh(complx	a)
		{
			double	x, y;
			complx	d;
			x = a.x;
			y = a.y;
			d.x = cos(y) * sinh(x);
			d.y = sin(y) * cosh(x);
			return	d;
		}

		friend	complx	cosh(complx	a)
		{
			double	x, y;
			complx	d;
			x = a.x ;
			y = a.y ;
			d.x = cos(y) * cosh(x);
			d.y = sin(y) * sinh(x);
			return	d;
		}

		friend	complx	tanh(complx	a)
		{
			return	sinh(a) / cosh(a);
		}

		friend	complx	asinh(complx	a)
		{
			return	log(a + sqrt(a * a + 1));
		}

		friend	complx	acosh(complx	a)
		{
			return	log(a + sqrt(a * a - 1));
		}

		friend	complx	atanh(complx	a)
		{
			return	log((1+a)/(1-a)) / 2;
		}

		friend	complx	asin(complx	a)
		{
		complx	c, d, e(0,1);
			c.x = -a.y;
			c.y =  a.x;
			d = - asinh(c) * e;
			while(d.x > M_PI_2)	d.x -= M_2_PI;
			while(d.x < -M_PI_2)	d.x += M_2_PI;
			if(d.x > M_PI_2)	d.x = M_PI - d.x;
			if(!rad)	d.x *= 180. / M_PI;
			return	d;
		}

		friend	complx	acos(complx	a)
		{
		complx	d, e(0,1);
			d = acosh(a) * e;
			while(d.x > M_PI)	d.x -= M_2_PI;
			while(d.x < 0)	d.x += M_2_PI;
			if(d.x > M_PI)	d.x = M_2_PI - d.x;
			if(!rad)	d.x *= 180. / M_PI;
			return	d;
		}

		friend	complx	atan(complx	a)
		{
		complx	c, d, e(0,1);
			c.x = -a.y;
			c.y =  a.x;
			d = - atanh(c) * e;
			while(d.x > M_PI_2)	d.x -= M_2_PI;
			while(d.x < -M_PI_2)	d.x += M_2_PI;
			if(d.x > M_PI_2)	d.x = d.x - M_PI;
			if(!rad)	d.x *= 180. / M_PI;
			return	d;
		}

		friend	complx	pow(complx	a, complx	b )
		{	return	exp(b * log(a));	}

		friend	complx	sqrt(complx	a)
		{
			complx	c, d;
			if((fabs(a.x) < min_num) && (fabs(a.y) < min_num))	return	complx(0.0, 0.0);
			c = log(a)/2.;	d = exp(c);
			return	complx(d.x, d.y);
		}

		friend	complx	_int(complx	a)
		{
			a.x = floor( a.x );
			a.y = floor( a.y );
			return a;
		}

		friend	complx	ceil(complx	a)
		{
			a.x = ceil( a.x );
			a.y = ceil( a.y );
			return a;
		}

		friend	complx	badr(complx	a)
		{
		double	x = a.x, h, fx = 0, X, FF;
		int			i, n = 1000;
			h = x / (double)n;	X = -h;
			for(i = 0; i <= n; i++){
				X += h;	FF = exp(-X*X/2.);
				if((i == 0) || (i == n)){
					fx += FF;	continue;
				}
				if(i & 1)	fx += 4. * FF;
				else			fx += 2. * FF;
			}
			fx *= (h / 3.) / sqrt(2. * (double)M_PI);
			fx = (1. - 2. * fx) * 100.;
			a.x = fx;
			a.y = 0;
			return a;
		}

		friend	complx	defl(complx	a)
		{
		double	p = a.x, x1 = 0, x2 = 4.0, x3, p1;
		int			i;
		complx	er;
			for(i = 0; i < 20; i++){
				x3 = (x1 + x2) / 2.;
				er = badr(complx(x3, 0));	p1 = er.x;
				if(p1 > p)	x1 = x3;
				else				x2 = x3;
			}
			a.x = x1;
			a.y = 0;
			return a;
		}

		friend	complx	estn(complx	a, complx b)
		{
		complx	c = defl(a);
		double	r = 4. / c.x;
			a.x = r * b.x;
			a.y = 0;
			return a;
		}

		friend	complx	build(complx	x, complx	y )
		{
		complx	c;
			c.x = x.x; c.y = y.x;
			return	c;
		}

		friend	complx	recip(complx	a)
		{
		double	c, d, e;
			if(a.x == 0 && a.y == 0){e=0; d=1/e;return d;}
			d = (a.x >= 0)?a.x:-a.x;	e = (a.y >= 0)?a.y:-a.y;
			if((d > max_num) || (e > max_num)){
				c = min_num2;
				return	complx(c, -c);
			}
			if((d < min_num) && (e < min_num)){
				c = max_num2;
				return	complx(c, -c);
			}
			return	1. / a;
		}

		friend	complx	swap(complx	x )
		{
		complx	c;
			c.x = x.y; c.y = x.x;
			return	c;
		}

		friend	complx	cswap(complx	x )
		{
		complx	c;
			c.x = x.y; c.y = -x.x;
			return	c;
		}

		friend	complx	swapc(complx	x )
		{
		complx	c;
			c.x = -x.y; c.y = x.x;
			return	c;
		}

		friend	complx	dbfrac(complx	x, complx	y )
		{
		double	d, e;
			if(y.x == 0 && y.y == 0){e=0; d=1/e;return d;}
			return	db(x)-db(y);
		}


};
